@@ -52,46 +52,47 @@ typedef struct grib_accessor_bufr_extract_datetime_subsets
 extern grib_accessor_class* grib_accessor_class_gen;
 
 static grib_accessor_class _grib_accessor_class_bufr_extract_datetime_subsets = {
-    &grib_accessor_class_gen,                            /* super                     */
-    "bufr_extract_datetime_subsets",                     /* name                      */
-    sizeof(grib_accessor_bufr_extract_datetime_subsets), /* size                      */
-    0,                                                   /* inited */
-    &init_class,                                         /* init_class */
-    &init,                                               /* init                      */
-    0,                                                   /* post_init                      */
-    0,                                                   /* free mem                       */
-    0,                                                   /* describes himself         */
-    0,                                                   /* get length of section     */
-    0,                                                   /* get length of string      */
-    0,                                                   /* get number of values      */
-    0,                                                   /* get number of bytes      */
-    0,                                                   /* get offset to bytes           */
-    &get_native_type,                                    /* get native type               */
-    0,                                                   /* get sub_section                */
-    0,                                                   /* grib_pack procedures long      */
-    0,                                                   /* grib_pack procedures long      */
-    &pack_long,                                          /* grib_pack procedures long      */
-    0,                                                   /* grib_unpack procedures long    */
-    0,                                                   /* grib_pack procedures double    */
-    0,                                                   /* grib_unpack procedures double  */
-    0,                                                   /* grib_pack procedures string    */
-    0,                                                   /* grib_unpack procedures string  */
-    0,                                                   /* grib_pack array procedures string    */
-    0,                                                   /* grib_unpack array procedures string  */
-    0,                                                   /* grib_pack procedures bytes     */
-    0,                                                   /* grib_unpack procedures bytes   */
-    0,                                                   /* pack_expression */
-    0,                                                   /* notify_change   */
-    0,                                                   /* update_size   */
-    0,                                                   /* preferred_size   */
-    0,                                                   /* resize   */
-    0,                                                   /* nearest_smaller_value */
-    0,                                                   /* next accessor    */
-    0,                                                   /* compare vs. another accessor   */
-    0,                                                   /* unpack only ith value          */
-    0,                                                   /* unpack a subarray         */
-    0,                                                   /* clear          */
-    0,                                                   /* clone accessor          */
+    &grib_accessor_class_gen,                      /* super */
+    "bufr_extract_datetime_subsets",                      /* name */
+    sizeof(grib_accessor_bufr_extract_datetime_subsets),  /* size */
+    0,                           /* inited */
+    &init_class,                 /* init_class */
+    &init,                       /* init */
+    0,                  /* post_init */
+    0,                    /* free mem */
+    0,                       /* describes himself */
+    0,                /* get length of section */
+    0,              /* get length of string */
+    0,                /* get number of values */
+    0,                 /* get number of bytes */
+    0,                /* get offset to bytes */
+    &get_native_type,            /* get native type */
+    0,                /* get sub_section */
+    0,               /* grib_pack procedures long */
+    0,                 /* grib_pack procedures long */
+    &pack_long,                  /* grib_pack procedures long */
+    0,                /* grib_unpack procedures long */
+    0,                /* grib_pack procedures double */
+    0,              /* grib_unpack procedures double */
+    0,                /* grib_pack procedures string */
+    0,              /* grib_unpack procedures string */
+    0,          /* grib_pack array procedures string */
+    0,        /* grib_unpack array procedures string */
+    0,                 /* grib_pack procedures bytes */
+    0,               /* grib_unpack procedures bytes */
+    0,            /* pack_expression */
+    0,              /* notify_change */
+    0,                /* update_size */
+    0,             /* preferred_size */
+    0,                     /* resize */
+    0,      /* nearest_smaller_value */
+    0,                       /* next accessor */
+    0,                    /* compare vs. another accessor */
+    0,      /* unpack only ith value */
+    0,  /* unpack a given set of elements */
+    0,     /* unpack a subarray */
+    0,                      /* clear */
+    0,                 /* clone accessor */
 };
 
 
@@ -100,36 +101,37 @@ grib_accessor_class* grib_accessor_class_bufr_extract_datetime_subsets = &_grib_
 
 static void init_class(grib_accessor_class* c)
 {
-    c->dump                   = (*(c->super))->dump;
-    c->next_offset            = (*(c->super))->next_offset;
-    c->string_length          = (*(c->super))->string_length;
-    c->value_count            = (*(c->super))->value_count;
-    c->byte_count             = (*(c->super))->byte_count;
-    c->byte_offset            = (*(c->super))->byte_offset;
-    c->sub_section            = (*(c->super))->sub_section;
-    c->pack_missing           = (*(c->super))->pack_missing;
-    c->is_missing             = (*(c->super))->is_missing;
-    c->unpack_long            = (*(c->super))->unpack_long;
-    c->pack_double            = (*(c->super))->pack_double;
-    c->unpack_double          = (*(c->super))->unpack_double;
-    c->pack_string            = (*(c->super))->pack_string;
-    c->unpack_string          = (*(c->super))->unpack_string;
-    c->pack_string_array      = (*(c->super))->pack_string_array;
-    c->unpack_string_array    = (*(c->super))->unpack_string_array;
-    c->pack_bytes             = (*(c->super))->pack_bytes;
-    c->unpack_bytes           = (*(c->super))->unpack_bytes;
-    c->pack_expression        = (*(c->super))->pack_expression;
-    c->notify_change          = (*(c->super))->notify_change;
-    c->update_size            = (*(c->super))->update_size;
-    c->preferred_size         = (*(c->super))->preferred_size;
-    c->resize                 = (*(c->super))->resize;
-    c->nearest_smaller_value  = (*(c->super))->nearest_smaller_value;
-    c->next                   = (*(c->super))->next;
-    c->compare                = (*(c->super))->compare;
-    c->unpack_double_element  = (*(c->super))->unpack_double_element;
-    c->unpack_double_subarray = (*(c->super))->unpack_double_subarray;
-    c->clear                  = (*(c->super))->clear;
-    c->make_clone             = (*(c->super))->make_clone;
+    c->dump    =    (*(c->super))->dump;
+    c->next_offset    =    (*(c->super))->next_offset;
+    c->string_length    =    (*(c->super))->string_length;
+    c->value_count    =    (*(c->super))->value_count;
+    c->byte_count    =    (*(c->super))->byte_count;
+    c->byte_offset    =    (*(c->super))->byte_offset;
+    c->sub_section    =    (*(c->super))->sub_section;
+    c->pack_missing    =    (*(c->super))->pack_missing;
+    c->is_missing    =    (*(c->super))->is_missing;
+    c->unpack_long    =    (*(c->super))->unpack_long;
+    c->pack_double    =    (*(c->super))->pack_double;
+    c->unpack_double    =    (*(c->super))->unpack_double;
+    c->pack_string    =    (*(c->super))->pack_string;
+    c->unpack_string    =    (*(c->super))->unpack_string;
+    c->pack_string_array    =    (*(c->super))->pack_string_array;
+    c->unpack_string_array    =    (*(c->super))->unpack_string_array;
+    c->pack_bytes    =    (*(c->super))->pack_bytes;
+    c->unpack_bytes    =    (*(c->super))->unpack_bytes;
+    c->pack_expression    =    (*(c->super))->pack_expression;
+    c->notify_change    =    (*(c->super))->notify_change;
+    c->update_size    =    (*(c->super))->update_size;
+    c->preferred_size    =    (*(c->super))->preferred_size;
+    c->resize    =    (*(c->super))->resize;
+    c->nearest_smaller_value    =    (*(c->super))->nearest_smaller_value;
+    c->next    =    (*(c->super))->next;
+    c->compare    =    (*(c->super))->compare;
+    c->unpack_double_element    =    (*(c->super))->unpack_double_element;
+    c->unpack_double_element_set    =    (*(c->super))->unpack_double_element_set;
+    c->unpack_double_subarray    =    (*(c->super))->unpack_double_subarray;
+    c->clear    =    (*(c->super))->clear;
+    c->make_clone    =    (*(c->super))->make_clone;
 }
 
 /* END_CLASS_IMP */
@@ -155,7 +157,8 @@ static int get_native_type(grib_accessor* a)
 /* Convert input date to Julian number. If date is invalid, return -1 */
 static double date_to_julian(long year, long month, long day, long hour, long minute, double second)
 {
-    double result = 0;
+    double result = 0; /* Julian number in units of days */
+
     /* For validating the date/time, we specify seconds as an integer */
     long lSecond = (long)second;
     grib_datetime_to_julian(year, month, day, hour, minute, lSecond, &result);
@@ -205,7 +208,7 @@ static int build_long_array(grib_context* c, grib_handle* h, int compressed,
     }
     else {
         /* uncompressed */
-        char keystr[20] = {0,};
+        char keystr[32] = {0,};
         size_t values_len = 0;
         for (i = 0; i < numberOfSubsets; ++i) {
             long lVal = 0;
@@ -243,12 +246,12 @@ static int select_datetime(grib_accessor* a)
     grib_iarray* subsets;
     long* subsets_ar   = 0;
     size_t nsubsets    = 0;
-    char yearstr[20]   = "year";
-    char monthstr[20]  = "month";
-    char daystr[20]    = "day";
-    char hourstr[20]   = "hour";
-    char minutestr[20] = "minute";
-    char secondstr[20] = "second";
+    char yearstr[32]   = "year";
+    char monthstr[32]  = "month";
+    char daystr[32]    = "day";
+    char hourstr[32]   = "hour";
+    char minutestr[32] = "minute";
+    char secondstr[32] = "second";
 
     ret = grib_get_long(h, "compressedData", &compressed);
     if (ret)
@@ -381,6 +384,7 @@ static int select_datetime(grib_accessor* a)
     if (ret)
         secondStart = 0;
     sprintf(start_str, "%04ld/%02ld/%02ld %02ld:%02ld:%02ld", yearStart, monthStart, dayStart, hourStart, minuteStart, secondStart);
+    if (c->debug) fprintf(stderr, "ECCODES DEBUG bufr_extract_datetime_subsets: start   =%s\n", start_str);
     julianStart = date_to_julian(yearStart, monthStart, dayStart, hourStart, minuteStart, secondStart);
     if (julianStart == -1) {
         grib_context_log(c, GRIB_LOG_ERROR, "Invalid start date/time: %s", start_str);
@@ -406,6 +410,7 @@ static int select_datetime(grib_accessor* a)
     if (ret)
         secondEnd = 0;
     sprintf(end_str, "%04ld/%02ld/%02ld %02ld:%02ld:%02ld", yearEnd, monthEnd, dayEnd, hourEnd, minuteEnd, secondEnd);
+    if (c->debug) fprintf(stderr, "ECCODES DEBUG bufr_extract_datetime_subsets: end     =%s\n", end_str);
     julianEnd = date_to_julian(yearEnd, monthEnd, dayEnd, hourEnd, minuteEnd, secondEnd);
     if (julianEnd == -1) {
         grib_context_log(c, GRIB_LOG_ERROR, "Invalid end date/time: %s", end_str);
@@ -418,7 +423,12 @@ static int select_datetime(grib_accessor* a)
     }
 
     for (i = 0; i < numberOfSubsets; i++) {
+        if (second[i] == GRIB_MISSING_DOUBLE) {
+            fprintf(stderr, "ECCODES WARNING: bufr_extract_datetime_subsets: Key '%s' is missing! Using zero instead\n", secondstr);
+            second[i] = 0;
+        }
         sprintf(datetime_str, "%04ld/%02ld/%02ld %02ld:%02ld:%.3f", year[i], month[i], day[i], hour[i], minute[i], second[i]);
+        if (c->debug) fprintf(stderr, "ECCODES DEBUG bufr_extract_datetime_subsets: datetime=%s\n", datetime_str);
         julianDT = date_to_julian(year[i], month[i], day[i], hour[i], minute[i], second[i]);
         if (julianDT == -1) {
             grib_context_log(c, GRIB_LOG_ERROR, "Invalid date/time: %s", datetime_str);
@@ -466,14 +476,9 @@ static int select_datetime(grib_accessor* a)
 
 static int pack_long(grib_accessor* a, const long* val, size_t* len)
 {
-    int err = 0;
     /*grib_accessor_bufr_extract_datetime_subsets *self =(grib_accessor_bufr_extract_datetime_subsets*)a;*/
 
     if (*len == 0)
         return GRIB_SUCCESS;
-    err = select_datetime(a);
-    if (err)
-        return err;
-
-    return err;
+    return select_datetime(a);
 }
