@@ -42,24 +42,23 @@ or edit "dumper.class" and rerun ./make_class.pl
 
 */
 
-static void init_class(grib_dumper_class*);
-static int init(grib_dumper* d);
-static int destroy(grib_dumper*);
-static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_bits(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_bytes(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_values(grib_dumper* d, grib_accessor* a);
-static void dump_label(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accessors* block);
-static void header(grib_dumper*, grib_handle*);
-static void footer(grib_dumper*, grib_handle*);
+static void init_class      (grib_dumper_class*);
+static int init            (grib_dumper* d);
+static int destroy         (grib_dumper*);
+static void dump_long       (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_bits       (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_double     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_string     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_string_array     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_bytes      (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_values     (grib_dumper* d, grib_accessor* a);
+static void dump_label      (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_section    (grib_dumper* d, grib_accessor* a,grib_block_of_accessors* block);
+static void header         (grib_dumper*,grib_handle*);
+static void footer         (grib_dumper*,grib_handle*);
 
-typedef struct grib_dumper_bufr_decode_python
-{
-    grib_dumper dumper;
+typedef struct grib_dumper_bufr_decode_python {
+    grib_dumper          dumper;  
     /* Members defined in bufr_decode_python */
     long section_offset;
     long empty;
@@ -71,24 +70,24 @@ typedef struct grib_dumper_bufr_decode_python
 
 
 static grib_dumper_class _grib_dumper_class_bufr_decode_python = {
-    0,                                      /* super                     */
-    "bufr_decode_python",                   /* name                      */
-    sizeof(grib_dumper_bufr_decode_python), /* size                      */
-    0,                                      /* inited */
-    &init_class,                            /* init_class */
-    &init,                                  /* init                      */
-    &destroy,                               /* free mem                       */
-    &dump_long,                             /* dump long         */
-    &dump_double,                           /* dump double    */
-    &dump_string,                           /* dump string    */
-    &dump_string_array,                     /* dump string array   */
-    &dump_label,                            /* dump labels  */
-    &dump_bytes,                            /* dump bytes  */
-    &dump_bits,                             /* dump bits   */
-    &dump_section,                          /* dump section      */
-    &dump_values,                           /* dump values   */
-    &header,                                /* header   */
-    &footer,                                /* footer   */
+    0,                              /* super                     */
+    "bufr_decode_python",                              /* name                      */
+    sizeof(grib_dumper_bufr_decode_python),     /* size                      */
+    0,                                   /* inited */
+    &init_class,                         /* init_class */
+    &init,                               /* init                      */
+    &destroy,                            /* free mem                       */
+    &dump_long,                          /* dump long         */
+    &dump_double,                        /* dump double    */
+    &dump_string,                        /* dump string    */
+    &dump_string_array,                        /* dump string array   */
+    &dump_label,                         /* dump labels  */
+    &dump_bytes,                         /* dump bytes  */
+    &dump_bits,                          /* dump bits   */
+    &dump_section,                       /* dump section      */
+    &dump_values,                        /* dump values   */
+    &header,                             /* header   */
+    &footer,                             /* footer   */
 };
 
 grib_dumper_class* grib_dumper_class_bufr_decode_python = &_grib_dumper_class_bufr_decode_python;
@@ -109,14 +108,6 @@ GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
     return (*a == 0 && *b == 0) ? 0 : 1;
 }
 
-typedef struct string_count string_count;
-struct string_count
-{
-    char* value;
-    int count;
-    string_count* next;
-};
-
 static int depth = 0;
 
 static void init_class(grib_dumper_class* c) {}
@@ -124,7 +115,7 @@ static void init_class(grib_dumper_class* c) {}
 static int init(grib_dumper* d)
 {
     grib_dumper_bufr_decode_python* self = (grib_dumper_bufr_decode_python*)d;
-    grib_context* c                      = d->handle->context;
+    grib_context* c                      = d->context;
     self->section_offset                 = 0;
     self->empty                          = 1;
     d->count                             = 1;
@@ -140,7 +131,7 @@ static int destroy(grib_dumper* d)
     grib_dumper_bufr_decode_python* self = (grib_dumper_bufr_decode_python*)d;
     grib_string_list* next               = self->keys;
     grib_string_list* cur                = NULL;
-    grib_context* c                      = d->handle->context;
+    grib_context* c                      = d->context;
     while (next) {
         cur  = next;
         next = next->next;
@@ -502,12 +493,13 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
     (void)err; /* TODO */
 }
 
+#define MAX_STRING_SIZE 4096
 static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
 {
     grib_dumper_bufr_decode_python* self = (grib_dumper_bufr_decode_python*)d;
-    char* value                          = NULL;
+    char value[MAX_STRING_SIZE] = {0,}; /* See ECC-710 */
     char* p                              = NULL;
-    size_t size                          = 0;
+    size_t size                          = MAX_STRING_SIZE;
     grib_context* c                      = a->context;
     int r = 0, err = 0;
     grib_handle* h = grib_handle_of_accessor(a);
@@ -515,23 +507,14 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
     if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
         return;
 
-    _grib_get_string_length(a, &size);
-    if (size == 0)
-        return;
-
-    value = (char*)grib_context_malloc_clear(c, size);
-    if (!value) {
-        grib_context_log(c, GRIB_LOG_FATAL, "unable to allocate %d bytes", (int)size);
-        return;
-    }
-
     self->empty = 0;
 
     err = grib_unpack_string(a, value, &size);
     p   = value;
     r   = compute_bufr_key_rank(h, self->keys, a->name);
-    if (grib_is_missing_string(a, (unsigned char*)value, size))
+    if (grib_is_missing_string(a, (unsigned char*)value, size)) {
         return;
+    }
 
     while (*p) {
         if (!isprint(*p))
@@ -565,7 +548,6 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
         depth -= 2;
     }
 
-    grib_context_free(c, value);
     (void)err; /* TODO */
 }
 
