@@ -42,24 +42,23 @@ or edit "dumper.class" and rerun ./make_class.pl
 
 */
 
-static void init_class(grib_dumper_class*);
-static int init(grib_dumper* d);
-static int destroy(grib_dumper*);
-static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_bits(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_bytes(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_values(grib_dumper* d, grib_accessor* a);
-static void dump_label(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accessors* block);
-static void header(grib_dumper*, grib_handle*);
-static void footer(grib_dumper*, grib_handle*);
+static void init_class      (grib_dumper_class*);
+static int init            (grib_dumper* d);
+static int destroy         (grib_dumper*);
+static void dump_long       (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_bits       (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_double     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_string     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_string_array     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_bytes      (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_values     (grib_dumper* d, grib_accessor* a);
+static void dump_label      (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_section    (grib_dumper* d, grib_accessor* a,grib_block_of_accessors* block);
+static void header         (grib_dumper*,grib_handle*);
+static void footer         (grib_dumper*,grib_handle*);
 
-typedef struct grib_dumper_bufr_encode_C
-{
-    grib_dumper dumper;
+typedef struct grib_dumper_bufr_encode_C {
+    grib_dumper          dumper;  
     /* Members defined in bufr_encode_C */
     long section_offset;
     long empty;
@@ -71,24 +70,24 @@ typedef struct grib_dumper_bufr_encode_C
 
 
 static grib_dumper_class _grib_dumper_class_bufr_encode_C = {
-    0,                                 /* super                     */
-    "bufr_encode_C",                   /* name                      */
-    sizeof(grib_dumper_bufr_encode_C), /* size                      */
-    0,                                 /* inited */
-    &init_class,                       /* init_class */
-    &init,                             /* init                      */
-    &destroy,                          /* free mem                       */
-    &dump_long,                        /* dump long         */
-    &dump_double,                      /* dump double    */
-    &dump_string,                      /* dump string    */
-    &dump_string_array,                /* dump string array   */
-    &dump_label,                       /* dump labels  */
-    &dump_bytes,                       /* dump bytes  */
-    &dump_bits,                        /* dump bits   */
-    &dump_section,                     /* dump section      */
-    &dump_values,                      /* dump values   */
-    &header,                           /* header   */
-    &footer,                           /* footer   */
+    0,                              /* super                     */
+    "bufr_encode_C",                              /* name                      */
+    sizeof(grib_dumper_bufr_encode_C),     /* size                      */
+    0,                                   /* inited */
+    &init_class,                         /* init_class */
+    &init,                               /* init                      */
+    &destroy,                            /* free mem                       */
+    &dump_long,                          /* dump long         */
+    &dump_double,                        /* dump double    */
+    &dump_string,                        /* dump string    */
+    &dump_string_array,                        /* dump string array   */
+    &dump_label,                         /* dump labels  */
+    &dump_bytes,                         /* dump bytes  */
+    &dump_bits,                          /* dump bits   */
+    &dump_section,                       /* dump section      */
+    &dump_values,                        /* dump values   */
+    &header,                             /* header   */
+    &footer,                             /* footer   */
 };
 
 grib_dumper_class* grib_dumper_class_bufr_encode_C = &_grib_dumper_class_bufr_encode_C;
@@ -109,14 +108,6 @@ GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
     return (*a == 0 && *b == 0) ? 0 : 1;
 }
 
-typedef struct string_count string_count;
-struct string_count
-{
-    char* value;
-    int count;
-    string_count* next;
-};
-
 static int depth = 0;
 
 static void init_class(grib_dumper_class* c) {}
@@ -124,7 +115,7 @@ static void init_class(grib_dumper_class* c) {}
 static int init(grib_dumper* d)
 {
     grib_dumper_bufr_encode_C* self = (grib_dumper_bufr_encode_C*)d;
-    grib_context* c                 = d->handle->context;
+    grib_context* c                 = d->context;
     self->section_offset            = 0;
     self->empty                     = 1;
     d->count                        = 1;
@@ -140,7 +131,7 @@ static int destroy(grib_dumper* d)
     grib_dumper_bufr_encode_C* self = (grib_dumper_bufr_encode_C*)d;
     grib_string_list* next          = self->keys;
     grib_string_list* cur           = NULL;
-    grib_context* c                 = d->handle->context;
+    grib_context* c                 = d->context;
     while (next) {
         cur  = next;
         next = next->next;
@@ -345,6 +336,11 @@ static void dump_values_attribute(grib_dumper* d, grib_accessor* a, const char*
     (void)err; /* TODO */
 }
 
+static int is_hidden(grib_accessor* a)
+{
+    return ( (a->flags & GRIB_ACCESSOR_FLAG_HIDDEN) != 0 );
+}
+
 static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
 {
     grib_dumper_bufr_encode_C* self = (grib_dumper_bufr_encode_C*)d;
@@ -360,8 +356,12 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
     grib_handle* h                  = grib_handle_of_accessor(a);
     int doing_unexpandedDescriptors = 0;
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0)
-        return;
+    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0) { /* key does not have the dump attribute */
+        int skip = 1;
+        /* See ECC-1107 */
+        if (!is_hidden(a) && strcmp(a->name, "messageLength") == 0) skip = 0;
+        if (skip) return;
+    }
 
     doing_unexpandedDescriptors = (strcmp(a->name, "unexpandedDescriptors") == 0);
     grib_value_count(a, &count);
@@ -665,6 +665,7 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
         depth -= 2;
     }
 
+    for (i = 0; i < size; i++) grib_context_free(c, values[i]);
     grib_context_free(c, values);
     (void)err; /* TODO */
 }
@@ -684,18 +685,8 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
     if (size == 0)
         return;
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0) {
-        /* ECC-356: Solution for the special local section key 'keyMore' and its alias 'ident' */
-        int skip = 1;
-        if ((a->flags & GRIB_ACCESSOR_FLAG_HIDDEN) != 0) {
-            if (strcmp(a->name, "keyMore") == 0 && grib_is_defined(h, "ls.ident")) {
-                skip     = 0;
-                acc_name = "ident";
-            }
-        }
-        if (skip)
-            return;
-    }
+    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
+        return;
 
     value = (char*)grib_context_malloc_clear(c, size);
     if (!value) {
@@ -714,7 +705,9 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
 
     while (*p) {
         if (!isprint(*p))
-            *p = '.';
+            *p = '?';
+        if (*p == '"')
+            *p = '\''; /* ECC-1401 */
         p++;
     }
 
@@ -898,7 +891,7 @@ static void header(grib_dumper* d, grib_handle* h)
 
     fprintf(self->dumper.out, "  h = codes_bufr_handle_new_from_samples(NULL, sampleName);\n");
     fprintf(self->dumper.out, "  if (h == NULL) {\n");
-    fprintf(self->dumper.out, "    fprintf(stderr, \"ERROR creating BUFR from %%s\\n\", sampleName);\n");
+    fprintf(self->dumper.out, "    fprintf(stderr, \"ERROR: Failed to create BUFR from %%s\\n\", sampleName);\n");
     fprintf(self->dumper.out, "    return 1;\n");
     fprintf(self->dumper.out, "  }\n");
 }
@@ -915,16 +908,16 @@ static void footer(grib_dumper* d, grib_handle* h)
 
     /*fprintf(self->dumper.out,"  fout = fopen(\"outfile.bufr\", \"w\");");*/
     fprintf(self->dumper.out, "  if (!fout) {\n");
-    fprintf(self->dumper.out, "    fprintf(stderr, \"Failed to open (%s) output file.\\n\");\n", (d->count == 1 ? "create" : "append"));
+    fprintf(self->dumper.out, "    fprintf(stderr, \"ERROR: Failed to open output file 'outfile.bufr' for writing.\\n\");\n");
     fprintf(self->dumper.out, "    return 1;\n");
     fprintf(self->dumper.out, "  }\n");
     fprintf(self->dumper.out, "  CODES_CHECK(codes_get_message(h,&buffer,&size),0);\n");
     fprintf(self->dumper.out, "  if (fwrite(buffer,1,size,fout) != size) {\n");
-    fprintf(self->dumper.out, "    fprintf(stderr, \"Failed to write data.\\n\");\n");
+    fprintf(self->dumper.out, "    fprintf(stderr, \"ERROR: Failed to write data.\\n\");\n");
     fprintf(self->dumper.out, "    return 1;\n");
     fprintf(self->dumper.out, "  }\n");
     fprintf(self->dumper.out, "  if (fclose(fout)!=0) {\n");
-    fprintf(self->dumper.out, "    fprintf(stderr, \"Failed to close file handle.\\n\");\n");
+    fprintf(self->dumper.out, "    fprintf(stderr, \"ERROR: Failed to close output file handle.\\n\");\n");
     fprintf(self->dumper.out, "    return 1;\n");
     fprintf(self->dumper.out, "  }\n");
     fprintf(self->dumper.out, "  \n");
