@@ -42,24 +42,23 @@ or edit "dumper.class" and rerun ./make_class.pl
 
 */
 
-static void init_class(grib_dumper_class*);
-static int init(grib_dumper* d);
-static int destroy(grib_dumper*);
-static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_bits(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_bytes(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_values(grib_dumper* d, grib_accessor* a);
-static void dump_label(grib_dumper* d, grib_accessor* a, const char* comment);
-static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accessors* block);
-static void header(grib_dumper*, grib_handle*);
-static void footer(grib_dumper*, grib_handle*);
+static void init_class      (grib_dumper_class*);
+static int init            (grib_dumper* d);
+static int destroy         (grib_dumper*);
+static void dump_long       (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_bits       (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_double     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_string     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_string_array     (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_bytes      (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_values     (grib_dumper* d, grib_accessor* a);
+static void dump_label      (grib_dumper* d, grib_accessor* a,const char* comment);
+static void dump_section    (grib_dumper* d, grib_accessor* a,grib_block_of_accessors* block);
+static void header         (grib_dumper*,grib_handle*);
+static void footer         (grib_dumper*,grib_handle*);
 
-typedef struct grib_dumper_bufr_encode_python
-{
-    grib_dumper dumper;
+typedef struct grib_dumper_bufr_encode_python {
+    grib_dumper          dumper;  
     /* Members defined in bufr_encode_python */
     long section_offset;
     long empty;
@@ -71,24 +70,24 @@ typedef struct grib_dumper_bufr_encode_python
 
 
 static grib_dumper_class _grib_dumper_class_bufr_encode_python = {
-    0,                                      /* super                     */
-    "bufr_encode_python",                   /* name                      */
-    sizeof(grib_dumper_bufr_encode_python), /* size                      */
-    0,                                      /* inited */
-    &init_class,                            /* init_class */
-    &init,                                  /* init                      */
-    &destroy,                               /* free mem                       */
-    &dump_long,                             /* dump long         */
-    &dump_double,                           /* dump double    */
-    &dump_string,                           /* dump string    */
-    &dump_string_array,                     /* dump string array   */
-    &dump_label,                            /* dump labels  */
-    &dump_bytes,                            /* dump bytes  */
-    &dump_bits,                             /* dump bits   */
-    &dump_section,                          /* dump section      */
-    &dump_values,                           /* dump values   */
-    &header,                                /* header   */
-    &footer,                                /* footer   */
+    0,                              /* super                     */
+    "bufr_encode_python",                              /* name                      */
+    sizeof(grib_dumper_bufr_encode_python),     /* size                      */
+    0,                                   /* inited */
+    &init_class,                         /* init_class */
+    &init,                               /* init                      */
+    &destroy,                            /* free mem                       */
+    &dump_long,                          /* dump long         */
+    &dump_double,                        /* dump double    */
+    &dump_string,                        /* dump string    */
+    &dump_string_array,                        /* dump string array   */
+    &dump_label,                         /* dump labels  */
+    &dump_bytes,                         /* dump bytes  */
+    &dump_bits,                          /* dump bits   */
+    &dump_section,                       /* dump section      */
+    &dump_values,                        /* dump values   */
+    &header,                             /* header   */
+    &footer,                             /* footer   */
 };
 
 grib_dumper_class* grib_dumper_class_bufr_encode_python = &_grib_dumper_class_bufr_encode_python;
@@ -109,14 +108,6 @@ GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
     return (*a == 0 && *b == 0) ? 0 : 1;
 }
 
-typedef struct string_count string_count;
-struct string_count
-{
-    char* value;
-    int count;
-    string_count* next;
-};
-
 static int depth = 0;
 
 static void init_class(grib_dumper_class* c) {}
@@ -648,15 +639,13 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
     fprintf(self->dumper.out, "    \"%s\", )\n", values[i]);
 
     if (self->isLeaf == 0) {
+        char* prefix;
+        int dofree = 0;
+
         if ((r = compute_bufr_key_rank(h, self->keys, a->name)) != 0)
             fprintf(self->dumper.out, "    codes_set_array(ibufr, '#%d#%s', svalues)\n", r, a->name);
         else
             fprintf(self->dumper.out, "    codes_set_array(ibufr, '%s', svalues)\n", a->name);
-    }
-
-    if (self->isLeaf == 0) {
-        char* prefix;
-        int dofree = 0;
 
         if (r != 0) {
             prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name) + 10));
@@ -691,18 +680,8 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
     if (size == 0)
         return;
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0) {
-        /* ECC-356: Solution for the special local section key 'keyMore' and its alias 'ident' */
-        int skip = 1;
-        if ((a->flags & GRIB_ACCESSOR_FLAG_HIDDEN) != 0) {
-            if (strcmp(acc_name, "keyMore") == 0 && grib_is_defined(h, "ls.ident")) {
-                skip     = 0;
-                acc_name = "ident";
-            }
-        }
-        if (skip)
-            return;
-    }
+    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
+        return;
 
     value = (char*)grib_context_malloc_clear(c, size);
     if (!value) {
@@ -721,7 +700,7 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
 
     while (*p) {
         if (!isprint(*p))
-            *p = '.';
+            *p = '?';
         p++;
     }
 
