@@ -26,7 +26,7 @@
    IMPLEMENTS = init;next
    MEMBERS     =   double *las
    MEMBERS     =   double *los
-   MEMBERS     =   long nam
+   MEMBERS     =   long Nj
    END_CLASS_DEF
 
  */
@@ -42,39 +42,38 @@ or edit "iterator.class" and rerun ./make_class.pl
 */
 
 
-static void init_class(grib_iterator_class*);
+static void init_class              (grib_iterator_class*);
 
-static int init(grib_iterator* i, grib_handle*, grib_arguments*);
-static int next(grib_iterator* i, double* lat, double* lon, double* val);
-static int destroy(grib_iterator* i);
+static int init               (grib_iterator* i,grib_handle*,grib_arguments*);
+static int next               (grib_iterator* i, double *lat, double *lon, double *val);
+static int destroy            (grib_iterator* i);
 
 
-typedef struct grib_iterator_gaussian_reduced
-{
-    grib_iterator it;
+typedef struct grib_iterator_gaussian_reduced{
+  grib_iterator it;
     /* Members defined in gen */
     long carg;
     const char* missingValue;
     /* Members defined in gaussian_reduced */
-    double* las;
-    double* los;
-    long nam;
+    double *las;
+    double *los;
+    long Nj;
 } grib_iterator_gaussian_reduced;
 
 extern grib_iterator_class* grib_iterator_class_gen;
 
 static grib_iterator_class _grib_iterator_class_gaussian_reduced = {
-    &grib_iterator_class_gen,               /* super                     */
-    "gaussian_reduced",                     /* name                      */
-    sizeof(grib_iterator_gaussian_reduced), /* size of instance          */
-    0,                                      /* inited */
-    &init_class,                            /* init_class */
-    &init,                                  /* constructor               */
-    &destroy,                               /* destructor                */
-    &next,                                  /* Next Value                */
-    0,                                      /*  Previous Value           */
-    0,                                      /* Reset the counter         */
-    0,                                      /* has next values           */
+    &grib_iterator_class_gen,                    /* super                     */
+    "gaussian_reduced",                    /* name                      */
+    sizeof(grib_iterator_gaussian_reduced),/* size of instance          */
+    0,                           /* inited */
+    &init_class,                 /* init_class */
+    &init,                     /* constructor               */
+    &destroy,                  /* destructor                */
+    &next,                     /* Next Value                */
+    0,                 /*  Previous Value           */
+    0,                    /* Reset the counter         */
+    0,                 /* has next values           */
 };
 
 grib_iterator_class* grib_iterator_class_gaussian_reduced = &_grib_iterator_class_gaussian_reduced;
@@ -82,9 +81,9 @@ grib_iterator_class* grib_iterator_class_gaussian_reduced = &_grib_iterator_clas
 
 static void init_class(grib_iterator_class* c)
 {
-    c->previous = (*(c->super))->previous;
-    c->reset    = (*(c->super))->reset;
-    c->has_next = (*(c->super))->has_next;
+    c->previous    =    (*(c->super))->previous;
+    c->reset    =    (*(c->super))->reset;
+    c->has_next    =    (*(c->super))->has_next;
 }
 /* END_CLASS_IMP */
 
@@ -128,7 +127,7 @@ static size_t count_subarea_points(grib_handle* h, get_reduced_row_proc get_redu
 }
 
 /* Search for 'x' in the array 'xx' (the index of last element being 'n') and return index in 'j' */
-static void binary_search(double xx[], const unsigned long n, double x, long* j)
+static void binary_search(const double xx[], const unsigned long n, double x, long* j)
 {
     /*This routine works only on descending ordered arrays*/
 #define EPSILON 1e-3
@@ -164,8 +163,8 @@ static int iterate_reduced_gaussian_subarea_legacy(grib_iterator* iter, grib_han
     double d       = 0;
     long ilon_first, ilon_last, i;
     grib_iterator_gaussian_reduced* self = (grib_iterator_gaussian_reduced*)iter;
-    get_reduced_row_proc get_reduced_row = &grib_get_reduced_row;
-    get_reduced_row                      = &grib_get_reduced_row_legacy; /* legacy algorithm */
+    /*get_reduced_row_proc get_reduced_row = &grib_get_reduced_row;*/
+    get_reduced_row_proc get_reduced_row   = &grib_get_reduced_row_legacy; /* legacy algorithm */
 
     if (h->context->debug) {
         const size_t np = count_subarea_points(h, get_reduced_row, pl, plsize, lon_first, lon_last);
@@ -190,7 +189,7 @@ static int iterate_reduced_gaussian_subarea_legacy(grib_iterator* iter, grib_han
             if (iter->e >= iter->nv) {
                 size_t np = count_subarea_points(h, get_reduced_row, pl, plsize, lon_first, lon_last);
                 grib_context_log(h->context, GRIB_LOG_ERROR,
-                                 "Reduced Gaussian iterator (sub-area legacy). Num points=%ld, size(values)=%ld", np, iter->nv);
+                                 "Reduced Gaussian Geoiterator (sub-area legacy). Num points=%ld, size(values)=%ld", np, iter->nv);
                 return GRIB_WRONG_GRID;
             }
 
@@ -251,7 +250,7 @@ static int iterate_reduced_gaussian_subarea_algorithm2(grib_iterator* iter, grib
                 /* Only print error message on the second pass */
                 size_t np = count_subarea_points(h, get_reduced_row, pl, plsize, lon_first, lon_last);
                 grib_context_log(h->context, GRIB_LOG_ERROR,
-                                 "Reduced Gaussian iterator (sub-area). Num points=%ld, size(values)=%ld", np, iter->nv);
+                                 "Reduced Gaussian Geoiterator (sub-area). Num points=%ld, size(values)=%ld", np, iter->nv);
                 return GRIB_WRONG_GRID;
             }
             self->los[iter->e] = lon2;
