@@ -21,8 +21,8 @@
    IMPLEMENTS = init;destroy
    MEMBERS    = double   *las
    MEMBERS    = double   *los
-   MEMBERS    = long      nap
-   MEMBERS    = long      nam
+   MEMBERS    = long      Ni
+   MEMBERS    = long      Nj
    MEMBERS    = long iScansNegatively
    MEMBERS    = long isRotated
    MEMBERS    = double angleOfRotation
@@ -45,25 +45,24 @@ or edit "iterator.class" and rerun ./make_class.pl
 */
 
 
-static void init_class(grib_iterator_class*);
+static void init_class              (grib_iterator_class*);
 
-static int init(grib_iterator* i, grib_handle*, grib_arguments*);
-static int next(grib_iterator* i, double* lat, double* lon, double* val);
-static int previous(grib_iterator* ei, double* lat, double* lon, double* val);
-static int destroy(grib_iterator* i);
+static int init               (grib_iterator* i,grib_handle*,grib_arguments*);
+static int next               (grib_iterator* i, double *lat, double *lon, double *val);
+static int previous           (grib_iterator* ei, double *lat, double *lon, double *val);
+static int destroy            (grib_iterator* i);
 
 
-typedef struct grib_iterator_regular
-{
-    grib_iterator it;
+typedef struct grib_iterator_regular{
+  grib_iterator it;
     /* Members defined in gen */
     long carg;
     const char* missingValue;
     /* Members defined in regular */
-    double* las;
-    double* los;
-    long nap;
-    long nam;
+    double   *las;
+    double   *los;
+    long      Ni;
+    long      Nj;
     long iScansNegatively;
     long isRotated;
     double angleOfRotation;
@@ -76,17 +75,17 @@ typedef struct grib_iterator_regular
 extern grib_iterator_class* grib_iterator_class_gen;
 
 static grib_iterator_class _grib_iterator_class_regular = {
-    &grib_iterator_class_gen,      /* super                     */
-    "regular",                     /* name                      */
-    sizeof(grib_iterator_regular), /* size of instance          */
-    0,                             /* inited */
-    &init_class,                   /* init_class */
-    &init,                         /* constructor               */
-    &destroy,                      /* destructor                */
-    &next,                         /* Next Value                */
-    &previous,                     /*  Previous Value           */
-    0,                             /* Reset the counter         */
-    0,                             /* has next values           */
+    &grib_iterator_class_gen,                    /* super                     */
+    "regular",                    /* name                      */
+    sizeof(grib_iterator_regular),/* size of instance          */
+    0,                           /* inited */
+    &init_class,                 /* init_class */
+    &init,                     /* constructor               */
+    &destroy,                  /* destructor                */
+    &next,                     /* Next Value                */
+    &previous,                 /*  Previous Value           */
+    0,                    /* Reset the counter         */
+    0,                 /* has next values           */
 };
 
 grib_iterator_class* grib_iterator_class_regular = &_grib_iterator_class_regular;
@@ -94,8 +93,8 @@ grib_iterator_class* grib_iterator_class_regular = &_grib_iterator_class_regular
 
 static void init_class(grib_iterator_class* c)
 {
-    c->reset    = (*(c->super))->reset;
-    c->has_next = (*(c->super))->has_next;
+    c->reset    =    (*(c->super))->reset;
+    c->has_next    =    (*(c->super))->has_next;
 }
 /* END_CLASS_IMP */
 
@@ -109,8 +108,8 @@ static int next(grib_iterator* i, double* lat, double* lon, double* val)
 
     i->e++;
 
-    *lat = self->las[(long)floor(i->e / self->nap)];
-    *lon = self->los[(long)i->e % self->nap];
+    *lat = self->las[(long)floor(i->e / self->Ni)];
+    *lon = self->los[(long)i->e % self->Ni];
     *val = i->data[i->e];
 
     return 1;
@@ -122,8 +121,8 @@ static int previous(grib_iterator* i, double* lat, double* lon, double* val)
 
     if (i->e < 0)
         return 0;
-    *lat = self->las[(long)floor(i->e / self->nap)];
-    *lon = self->los[i->e % self->nap];
+    *lat = self->las[(long)floor(i->e / self->Ni)];
+    *lon = self->los[i->e % self->Ni];
     *val = i->data[i->e];
     i->e--;
 
@@ -163,12 +162,22 @@ static int init(grib_iterator* i, grib_handle* h, grib_arguments* args)
         return ret;
     if ((ret = grib_get_long_internal(h, s_Ni, &Ni)))
         return ret;
+    if (grib_is_missing(h, s_Ni, &ret) && ret == GRIB_SUCCESS) {
+        grib_context_log(h->context, GRIB_LOG_ERROR, "Key %s cannot be 'missing' for a regular grid!", s_Ni);
+        return GRIB_WRONG_GRID;
+    }
+
     if ((ret = grib_get_long_internal(h, s_Nj, &Nj)))
         return ret;
+    if (grib_is_missing(h, s_Nj, &ret) && ret == GRIB_SUCCESS) {
+        grib_context_log(h->context, GRIB_LOG_ERROR, "Key %s cannot be 'missing' for a regular grid!", s_Nj);
+        return GRIB_WRONG_GRID;
+    }
+
     if ((ret = grib_get_long_internal(h, s_iScansNeg, &self->iScansNegatively)))
         return ret;
 
-    /* GRIB-801: Careful of case with a single point! nap==1 */
+    /* GRIB-801: Careful of case with a single point! Ni==1 */
     if (Ni > 1) {
         /* Note: If first and last longitudes are equal I assume you wanna go round the globe */
         if (self->iScansNegatively) {
@@ -200,8 +209,8 @@ static int init(grib_iterator* i, grib_handle* h, grib_arguments* args)
         }*/
     }
 
-    self->nap = Ni;
-    self->nam = Nj;
+    self->Ni = Ni;
+    self->Nj = Nj;
 
     self->las = (double*)grib_context_malloc(h->context, Nj * sizeof(double));
     self->los = (double*)grib_context_malloc(h->context, Ni * sizeof(double));
@@ -210,6 +219,10 @@ static int init(grib_iterator* i, grib_handle* h, grib_arguments* args)
         self->los[loi] = lon1;
         lon1 += idir;
     }
+    /* ECC-1406: Due to rounding, errors can accumulate.
+     * So we ensure the last longitude is longitudeOfLastGridPointInDegrees
+    */
+    self->los[Ni-1] = lon2;
 
     return ret;
 }
