@@ -19,7 +19,7 @@ typedef enum
     eROUND_ANGLE_DOWN
 } RoundingPolicy;
 
-static void set_total_length(unsigned char* buffer, long* section_length, long* section_offset, int edition, size_t totalLength)
+static void set_total_length(unsigned char* buffer, long* section_length, const long* section_offset, int edition, size_t totalLength)
 {
     long off;
     switch (edition) {
@@ -429,6 +429,8 @@ static void print_values(grib_context* c,
     if (c->gribex_mode_on)
         fprintf(stderr, "ECCODES DEBUG grib_util: GRIBEX mode is turned on!\n");
 
+    fprintf(stderr, "ECCODES DEBUG grib_util: packing_spec->editionNumber = %ld\n",
+            packing_spec->editionNumber);
     fprintf(stderr, "ECCODES DEBUG grib_util: packing_spec->packing = %s\n",
             get_packing_spec_packing_name(packing_spec->packing));
     fprintf(stderr, "ECCODES DEBUG grib_util: packing_spec->packing_type = %s\n",
@@ -980,6 +982,16 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
         fprintf(stderr, "ECCODES DEBUG grib_util: input_decimal_scale_factor = %ld\n", input_decimal_scale_factor);
     }
 
+    /* ECC-1201
+       TODO: make sure input packing type is preserved */
+    if (packing_spec->packing == GRIB_UTIL_PACKING_SAME_AS_INPUT &&
+        packing_spec->packing_type == GRIB_UTIL_PACKING_TYPE_SAME_AS_INPUT)
+    {
+        if (STR_EQUAL(input_packing_type, "grid_ieee")) {
+            SET_STRING_VALUE("packingType", input_packing_type);
+        }
+    }
+
     /*if ( (*err=check_values(data_values, data_values_count))!=GRIB_SUCCESS ) {
         fprintf(stderr,"GRIB_UTIL_SET_SPEC: Data values check failed! %s\n", grib_get_error_message(*err));
         goto cleanup;
@@ -1026,7 +1038,8 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
                     setSecondOrder = 1;
                     break;
                 default:
-                    fprintf(stderr, "invalid packing_spec->packing_type = %ld\n", (long)packing_spec->packing_type);
+                    fprintf(stderr, "GRIB_UTIL_SET_SPEC: invalid packing_spec->packing_type (%ld)\n",
+                            (long)packing_spec->packing_type);
                     *err = GRIB_INTERNAL_ERROR;
                     goto cleanup;
                     break;
@@ -1050,7 +1063,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
                 break;
 
             default:
-                fprintf(stderr, "invalid packing_spec->accuracy = %ld\n", (long)packing_spec->accuracy);
+                fprintf(stderr, "GRIB_UTIL_SET_SPEC: invalid packing_spec->accuracy (%ld)\n", (long)packing_spec->accuracy);
                 *err = GRIB_INTERNAL_ERROR;
                 goto cleanup;
                 break;
@@ -1067,7 +1080,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
         }
 
         if ((*err = grib_set_values(h, values, count)) != 0) {
-            fprintf(stderr, "GRIB_UTIL_SET_SPEC: Cannot set values  %s\n", grib_get_error_message(*err));
+            fprintf(stderr, "GRIB_UTIL_SET_SPEC: Cannot set values: %s\n", grib_get_error_message(*err));
             for (i = 0; i < count; i++)
                 if (values[i].error) fprintf(stderr, " %s %s\n", values[i].name, grib_get_error_message(values[i].error));
             goto cleanup;
@@ -1132,16 +1145,14 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     } /* flags & GRIB_UTIL_SET_SPEC_FLAGS_ONLY_PACKING */
 
     grid_type = get_grid_type_name(spec->grid_type);
-    if (grid_type == NULL) {
-        fprintf(stderr, "GRIB_UTIL_SET_SPEC: Unknown grid type: %d\n", spec->grid_type);
+    if (!grid_type) {
+        fprintf(stderr, "GRIB_UTIL_SET_SPEC: Unknown spec.grid_type (%d)\n", spec->grid_type);
         *err = GRIB_NOT_IMPLEMENTED;
         return NULL;
     }
-
     SET_STRING_VALUE("gridType", grid_type);
 
     /* The "pl" is given from the template, but "section_copy" will take care of setting the right headers */
-
     {
         switch (spec->grid_type) {
             case GRIB_UTIL_GRID_SPEC_REDUCED_GG:
@@ -1157,10 +1168,9 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
             case GRIB_UTIL_GRID_SPEC_UNSTRUCTURED:
                 if (editionNumber == 1) { /* This grid type is not available in edition 1 */
                     if (h->context->debug == -1)
-                        fprintf(stderr,
-                                "ECCODES DEBUG grib_util: '%s' specified "
-                                "but input is GRIB1. Output must be a higher edition!\n",
-                                grid_type);
+                        fprintf(stderr, "ECCODES DEBUG grib_util: '%s' specified "
+                                    "but input is GRIB1. Output must be a higher edition!\n",
+                                    grid_type);
                     convertEditionEarlier = 1;
                 }
                 sprintf(sample_name, "GRIB%ld", editionNumber);
@@ -1174,7 +1184,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
 
         if (spec->pl && spec->grid_name) {
             /* Cannot have BOTH pl and grid name specified */
-            fprintf(stderr, "GRIB_UTIL_SET_SPEC: Cannot set BOTH pl and grid_name.\n");
+            fprintf(stderr, "GRIB_UTIL_SET_SPEC: Cannot set BOTH spec.pl and spec.grid_name!\n");
             goto cleanup;
         }
         if (spec->grid_name) {
@@ -1401,8 +1411,10 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
                     SET_STRING_VALUE("packingType", "grid_simple");
                 break;
             case GRIB_UTIL_PACKING_TYPE_GRID_COMPLEX:
-                if (strcmp(input_packing_type, "grid_complex") && !strcmp(input_packing_type, "grid_simple"))
+                if (!STR_EQUAL(input_packing_type, "grid_complex")) {
                     SET_STRING_VALUE("packingType", "grid_complex");
+                    convertEditionEarlier=1;
+                }
                 break;
             case GRIB_UTIL_PACKING_TYPE_JPEG:
                 /* Have to delay JPEG packing:
@@ -1417,11 +1429,11 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
                  * Reason 1: It is not available in GRIB1 and so we have to wait until we change edition
                  * Reason 2: It has to be done AFTER we set the data values
                  */
-                if (strcmp(input_packing_type, "grid_ccsds") && !strcmp(input_packing_type, "grid_simple"))
+                if (!STR_EQUAL(input_packing_type, "grid_ccsds"))
                     setCcsdsPacking = 1;
                 break;
             case GRIB_UTIL_PACKING_TYPE_IEEE:
-                if (strcmp(input_packing_type, "grid_ieee") && !strcmp(input_packing_type, "grid_simple"))
+                if ( !STR_EQUAL(input_packing_type, "grid_ieee") )
                     SET_STRING_VALUE("packingType", "grid_ieee");
                 break;
             case GRIB_UTIL_PACKING_TYPE_GRID_SECOND_ORDER:
@@ -1430,7 +1442,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
                 setSecondOrder = 1;
                 break;
             default:
-                fprintf(stderr, "invalid packing_spec->packing_type = %ld\n", (long)packing_spec->packing_type);
+                fprintf(stderr, "GRIB_UTIL_SET_SPEC: invalid packing_spec.packing_type (%ld)\n", (long)packing_spec->packing_type);
                 *err = GRIB_INTERNAL_ERROR;
                 goto cleanup;
                 break;
@@ -1465,8 +1477,17 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     switch (packing_spec->accuracy) {
         case GRIB_UTIL_ACCURACY_SAME_BITS_PER_VALUES_AS_INPUT: {
             long bitsPerValue = 0;
-            Assert(grib_get_long(h, "bitsPerValue", &bitsPerValue) == 0);
-            SET_LONG_VALUE("bitsPerValue", bitsPerValue);
+            if ((packing_spec->packing_type == GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE ||
+                 packing_spec->packing_type == GRIB_UTIL_PACKING_TYPE_CCSDS)      &&
+                strcmp(input_packing_type, "grid_ieee")==0)
+            {
+                SET_LONG_VALUE("bitsPerValue", 32);
+            }
+            else
+            {
+                Assert(grib_get_long(h, "bitsPerValue", &bitsPerValue) == 0);
+                SET_LONG_VALUE("bitsPerValue", bitsPerValue);
+            }
         } break;
 
         case GRIB_UTIL_ACCURACY_USE_PROVIDED_BITS_PER_VALUES:
@@ -1484,7 +1505,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
             break;
 
         default:
-            fprintf(stderr, "invalid packing_spec->accuracy = %ld\n", (long)packing_spec->accuracy);
+            fprintf(stderr, "GRIB_UTIL_SET_SPEC: invalid packing_spec.accuracy (%ld)\n", (long)packing_spec->accuracy);
             grib_handle_delete(h_sample);
             *err = GRIB_INTERNAL_ERROR;
             goto cleanup;
@@ -1525,24 +1546,25 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     /* GRIB-857: Set "pl" array if provided (For reduced Gaussian grids) */
     Assert(spec->pl_size >= 0);
     if (spec->pl && spec->pl_size == 0) {
-        fprintf(stderr, "pl array not NULL but pl_size == 0!\n");
+        fprintf(stderr, "GRIB_UTIL_SET_SPEC: pl array not NULL but pl_size == 0!\n");
         goto cleanup;
     }
     if (spec->pl_size > 0 && spec->pl == NULL) {
-        fprintf(stderr, "pl_size not zero but pl array == NULL!\n");
+        fprintf(stderr, "GRIB_UTIL_SET_SPEC: pl_size not zero but pl array == NULL!\n");
         goto cleanup;
     }
 
     if (spec->pl_size != 0 && (spec->grid_type == GRIB_UTIL_GRID_SPEC_REDUCED_GG || spec->grid_type == GRIB_UTIL_GRID_SPEC_REDUCED_ROTATED_GG)) {
         *err = grib_set_long_array(h_out, "pl", spec->pl, spec->pl_size);
         if (*err) {
-            fprintf(stderr, "SET_GRID_DATA_DESCRIPTION: Cannot set pl  %s\n", grib_get_error_message(*err));
+            fprintf(stderr, "GRIB_UTIL_SET_SPEC: Cannot set pl: %s\n", grib_get_error_message(*err));
             goto cleanup;
         }
         if (global_grid) {
             size_t sum = sum_of_pl_array(spec->pl, spec->pl_size);
             if (data_values_count != sum) {
-                fprintf(stderr, "invalid reduced gaussian grid: specified as global, data_values_count=%ld but sum of pl array=%ld\n",
+                fprintf(stderr, "GRIB_UTIL_SET_SPEC: invalid reduced gaussian grid: "
+                        "specified as global, data_values_count=%ld but sum of pl array=%ld\n",
                         (long)data_values_count, (long)sum);
                 *err = GRIB_WRONG_GRID;
                 goto cleanup;
@@ -1559,7 +1581,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     /* Apply adjustments to bounding box if needed */
     if (expandBoundingBox) {
         if ((*err = expand_bounding_box(h_out, values, count)) != 0) {
-            fprintf(stderr, "SET_GRID_DATA_DESCRIPTION: Cannot expand bounding box: %s\n", grib_get_error_message(*err));
+            fprintf(stderr, "GRIB_UTIL_SET_SPEC: Cannot expand bounding box: %s\n", grib_get_error_message(*err));
             if (h->context->write_on_fail)
                 grib_write_message(h_out, "error.grib", "w");
             goto cleanup;
@@ -1660,7 +1682,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
                 grib_set_string(h_out, "packingType", "grid_second_order", &slen);
                 *err = grib_set_double_array(h_out, "values", data_values, data_values_count);
                 if (*err != GRIB_SUCCESS) {
-                    fprintf(stderr, "GRIB_UTIL_SET_SPEC: setting data values failed! %s\n", grib_get_error_message(*err));
+                    fprintf(stderr, "GRIB_UTIL_SET_SPEC: setting data values failed: %s\n", grib_get_error_message(*err));
                     goto cleanup;
                 }
             }
@@ -1715,7 +1737,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     }
 
     if ((*err = check_geometry(h_out, spec, data_values_count, global_grid)) != GRIB_SUCCESS) {
-        fprintf(stderr, "GRIB_UTIL_SET_SPEC: Geometry check failed! %s\n", grib_get_error_message(*err));
+        fprintf(stderr, "GRIB_UTIL_SET_SPEC: Geometry check failed: %s\n", grib_get_error_message(*err));
         if (h->context->write_on_fail)
             grib_write_message(h_out, "error.grib", "w");
         goto cleanup;
@@ -1725,7 +1747,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
 #if 0
     if ( (*err = check_handle_against_spec(h_out, editionNumber, spec, global_grid)) != GRIB_SUCCESS) {
         grib_context* c=grib_context_get_default();
-        fprintf(stderr,"GRIB_UTIL_SET_SPEC: Geometry check failed! %s\n", grib_get_error_message(*err));
+        fprintf(stderr,"GRIB_UTIL_SET_SPEC: Geometry check failed: %s\n", grib_get_error_message(*err));
         if (editionNumber == 1) {
             fprintf(stderr,"Note: in GRIB edition 1 latitude and longitude values cannot be represented with sub-millidegree precision.\n");
         }
@@ -2194,6 +2216,48 @@ int grib_is_earth_oblate(grib_handle* h)
     return 0;
 }
 
+int grib_check_data_values_range(grib_handle* h, const double min_val, const double max_val)
+{
+    int result        = GRIB_SUCCESS;
+    grib_context* ctx = h->context;
+
+    if (!(min_val < DBL_MAX && min_val > -DBL_MAX)) {
+        grib_context_log(ctx, GRIB_LOG_ERROR, "Minimum value out of range: %g", min_val);
+        return GRIB_ENCODING_ERROR;
+    }
+    if (!(max_val < DBL_MAX && max_val > -DBL_MAX)) {
+        grib_context_log(ctx, GRIB_LOG_ERROR, "Maximum value out of range: %g", max_val);
+        return GRIB_ENCODING_ERROR;
+    }
+
+    /* Data Quality checks */
+    if (ctx->grib_data_quality_checks) {
+        result = grib_util_grib_data_quality_check(h, min_val, max_val);
+    }
+
+    return result;
+}
+
+/* Return true(1) if large constant fields are to be created, otherwise false(0) */
+int grib_producing_large_constant_fields(grib_handle* h, int edition)
+{
+    /* First check if the transient key is set */
+    grib_context* c                 = h->context;
+    long produceLargeConstantFields = 0;
+    if (grib_get_long(h, "produceLargeConstantFields", &produceLargeConstantFields) == GRIB_SUCCESS &&
+        produceLargeConstantFields != 0) {
+        return 1;
+    }
+
+    if (c->gribex_mode_on == 1 && edition == 1) {
+        return 1;
+    }
+
+    /* Finally check the environment variable via the context */
+    return c->large_constant_fields;
+}
+
+
 int grib_util_grib_data_quality_check(grib_handle* h, double min_val, double max_val)
 {
     int err                        = 0;
