@@ -9,10 +9,9 @@
  */
 #include <stdio.h>
 #include <stdlib.h>
-#include <time.h>
 #include <pthread.h>
-#include <assert.h>
-#include "grib_api.h"
+
+#include "grib_api_internal.h"
 
 #define NUM_THREADS 8
 #define FILES_PER_ITERATION 10
@@ -58,8 +57,8 @@ static int encode_file(char* input_file, char* output_file)
 
     FILE* in  = fopen(input_file, "rb");
     FILE* out = fopen(output_file, "wb");
-    assert(in);
-    assert(out);
+    Assert(in);
+    Assert(out);
 
     while ((source_handle = grib_handle_new_from_file(0, in, &err)) != NULL) {
         size_t size = 0, values_len = 0;
@@ -69,7 +68,7 @@ static int encode_file(char* input_file, char* output_file)
         double d, e;
 
         grib_handle* clone_handle = grib_handle_clone(source_handle);
-        assert(clone_handle);
+        Assert(clone_handle);
 
         GRIB_CHECK(grib_get_size(clone_handle, "values", &values_len), 0);
         values = (double*)malloc(values_len * sizeof(double));
@@ -87,6 +86,10 @@ static int encode_file(char* input_file, char* output_file)
             count++;
         }
         GRIB_CHECK(grib_set_long(clone_handle, "bitsPerValue", 16), 0);
+
+        /*GRIB_CHECK(grib_set_string(clone_handle, "packingType", "grid_ccsds", &str_len), 0);*/
+        /*GRIB_CHECK(grib_set_string(clone_handle, "packingType", "grid_simple", &str_len), 0);*/
+
         GRIB_CHECK(grib_set_double_array(clone_handle, "values", values, values_len), 0);
 
         /* get the coded message in a buffer */
@@ -114,7 +117,7 @@ void do_stuff(void* arg)
     int i;
 
     for (i = 0; i < FILES_PER_ITERATION; i++) {
-        sprintf(output_file, "temp.grib_encode_pthreads.out_%d-%d.grib", (int)number, i);
+        sprintf(output_file, "temp.grib_encode_pthreads_test.out_%d-%d.grib", (int)number, i);
         encode_file(INPUT_FILE, output_file);
     }
 }
