@@ -215,7 +215,7 @@ static int grib_tool_with_orderby(grib_runtime_options* options)
     grib_failed *failed = NULL, *p = NULL;
     grib_handle* h          = NULL;
     grib_tools_file* infile = options->infile;
-    char** filenames;
+    const char** filenames = NULL;
     int files_count    = 0;
     grib_fieldset* set = NULL;
     int i              = 0;
@@ -230,7 +230,7 @@ static int grib_tool_with_orderby(grib_runtime_options* options)
         infile = infile->next;
     }
 
-    filenames = (char**)grib_context_malloc_clear(c, files_count * sizeof(char*));
+    filenames = (const char**)grib_context_malloc_clear(c, files_count * sizeof(char*));
 
     infile = options->infile;
     for (i = 0; i < files_count; i++) {
@@ -297,7 +297,7 @@ static int grib_tool_with_orderby(grib_runtime_options* options)
         grib_handle_delete(h);
     }
 
-    if (set->size==0) fprintf(stderr, "no messages found in fieldset\n");
+    if (set->size==0) fprintf(stderr, "No messages found in fieldset\n");
     grib_tool_finalise_action(options);
 
     grib_fieldset_delete(set);
@@ -821,7 +821,7 @@ static int to_skip(grib_handle* h, grib_values* v, int* err)
             ret    = (lvalue == v->equal) ? 0 : 1;
             break;
         default:
-            fprintf(dump_file, "invalid type for %s\n", v->name);
+            fprintf(dump_file, "ERROR: Invalid type for %s\n", v->name);
             exit(1);
     }
 
@@ -849,8 +849,12 @@ void grib_skip_check(grib_runtime_options* options, grib_handle* h)
             options->skip = 1;
             while (v) {
                 if (!to_skip(h, v, &ret)) {
-                    if (!strcmp(v->name, "count") && !v->next)
+                    if (!strcmp(v->name, "count") && !v->next) {
+                        /* We have count=XX and nothing after that so we can
+                         * skip every other message after it
+                         */
                         options->skip_all = 1;
+                    }
                     options->skip = 0;
                     break;
                 }
@@ -900,8 +904,10 @@ static int is_valid_JSON_number(const char* input)
         p++;
 
     while (*p) {
-        if (*p == '.')
+        if (*p == '.') {
+            if (is_float) return 0; /*more than 1 dot*/
             is_float = 1;
+        }
         if (*p != '.' && !isdigit(*p))
             return 0;
         p++;
@@ -917,80 +923,6 @@ static int is_valid_JSON_number(const char* input)
     return 1;
 }
 
-static void get_value_for_key(grib_handle* h, const char* key_name, int key_type, char* value_str,
-                              const char* format, int fix_lsdate, int fix_lstime)
-{
-    int ret = 0, type = key_type;
-    double dvalue = 0;
-    long lvalue   = 0;
-    size_t len    = MAX_STRING_LEN;
-
-    if (grib_is_missing(h, key_name, &ret) && ret == GRIB_SUCCESS) {
-        sprintf(value_str, "MISSING");
-        return;
-    }
-    if (ret == GRIB_NOT_FOUND) {
-        sprintf(value_str, "not_found");
-        return;
-    }
-
-    if (type == GRIB_TYPE_UNDEFINED) {
-        ret = grib_get_native_type(h, key_name, &type);
-        if (ret != GRIB_SUCCESS) {
-            fprintf(dump_file, "Could not determine type for %s\n", key_name);
-            exit(1);
-        }
-    }
-
-    if (type == GRIB_TYPE_STRING) {
-        const char* pName = key_name;
-        /* ECC-707 */
-        if (fix_lsdate && strcmp(pName, "date") == 0) pName = "ls.date";
-        if (fix_lstime && strcmp(pName, "time") == 0) pName = "ls.time";
-        ret = grib_get_string(h, pName, value_str, &len);
-    }
-    else if (type == GRIB_TYPE_DOUBLE) {
-        ret = grib_get_double(h, key_name, &dvalue);
-        sprintf(value_str, format, dvalue);
-    }
-    else if (type == GRIB_TYPE_LONG) {
-        ret = grib_get_long(h, key_name, &lvalue);
-        sprintf(value_str, "%ld", lvalue);
-    }
-    else if (type == GRIB_TYPE_BYTES) {
-        ret = grib_get_string(h, key_name, value_str, &len);
-    }
-    else {
-        fprintf(dump_file, "invalid format option for %s\n", key_name);
-        exit(1);
-    }
-
-    if (ret != GRIB_SUCCESS) {
-        if (ret == GRIB_NOT_FOUND) {
-            sprintf(value_str, "not_found");
-        } else {
-            fprintf(dump_file, "Failed to get value for key %s\n", key_name);
-            exit(1);
-        }
-    }
-}
-
-/* See ECC-707 */
-static int fix_for_lsdate_needed(grib_handle* h)
-{
-    long lsdate_bug = 0;
-    int err         = grib_get_long(h, "lsdate_bug", &lsdate_bug);
-    if (!err && lsdate_bug == 1) return 1;
-    return 0;
-}
-static int fix_for_lstime_needed(grib_handle* h)
-{
-    long lstime_bug = 0;
-    int err         = grib_get_long(h, "lstime_bug", &lstime_bug);
-    if (!err && lstime_bug == 1) return 1;
-    return 0;
-}
-
 static int get_initial_element_of_array(grib_handle* h, const char* keyName, size_t num_vals, char* value)
 {
     int err = 0, type = 0;
@@ -1050,6 +982,86 @@ static int get_initial_element_of_array(grib_handle* h, const char* keyName, siz
     return GRIB_SUCCESS;
 }
 
+static void get_value_for_key(grib_handle* h, const char* key_name, int key_type, char* value_str,
+                              const char* format, int fix_lsdate, int fix_lstime)
+{
+    int ret = 0, type = key_type;
+    double dvalue = 0;
+    long lvalue   = 0;
+    size_t len    = MAX_STRING_LEN;
+
+    if (grib_is_missing(h, key_name, &ret) && ret == GRIB_SUCCESS) {
+        sprintf(value_str, "MISSING");
+        return;
+    }
+    if (ret == GRIB_NOT_FOUND) {
+        sprintf(value_str, "not_found");
+        return;
+    }
+
+    if (type == GRIB_TYPE_UNDEFINED) {
+        ret = grib_get_native_type(h, key_name, &type);
+        if (ret != GRIB_SUCCESS) {
+            fprintf(dump_file, "ERROR: Could not determine type for %s\n", key_name);
+            exit(1);
+        }
+    }
+
+    if (type == GRIB_TYPE_STRING) {
+        const char* pName = key_name;
+        size_t num_vals = 0;
+        /* ECC-707 */
+        if (fix_lsdate && strcmp(pName, "date") == 0) pName = "ls.date";
+        if (fix_lstime && strcmp(pName, "time") == 0) pName = "ls.time";
+        ret = grib_get_size(h, pName, &num_vals);
+        if (ret == GRIB_SUCCESS && num_vals > 1) { /* See ECC-278 */
+            ret = get_initial_element_of_array(h, pName, num_vals, value_str);
+        } else {
+            ret = grib_get_string(h, pName, value_str, &len);
+        }
+    }
+    else if (type == GRIB_TYPE_DOUBLE) {
+        ret = grib_get_double(h, key_name, &dvalue);
+        sprintf(value_str, format, dvalue);
+    }
+    else if (type == GRIB_TYPE_LONG) {
+        ret = grib_get_long(h, key_name, &lvalue);
+        sprintf(value_str, "%ld", lvalue);
+    }
+    else if (type == GRIB_TYPE_BYTES) {
+        ret = grib_get_string(h, key_name, value_str, &len);
+    }
+    else {
+        fprintf(dump_file, "ERROR: Invalid format option for %s\n", key_name);
+        exit(1);
+    }
+
+    if (ret != GRIB_SUCCESS) {
+        if (ret == GRIB_NOT_FOUND) {
+            sprintf(value_str, "not_found");
+        } else {
+            fprintf(dump_file, "ERROR: Failed to get value for key %s\n", key_name);
+            exit(1);
+        }
+    }
+}
+
+/* See ECC-707 */
+static int fix_for_lsdate_needed(grib_handle* h)
+{
+    long lsdate_bug = 0;
+    int err         = grib_get_long(h, "lsdate_bug", &lsdate_bug);
+    if (!err && lsdate_bug == 1) return 1;
+    return 0;
+}
+static int fix_for_lstime_needed(grib_handle* h)
+{
+    long lstime_bug = 0;
+    int err         = grib_get_long(h, "lstime_bug", &lstime_bug);
+    if (!err && lstime_bug == 1) return 1;
+    return 0;
+}
+
 void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
 {
     int i   = 0;
@@ -1092,8 +1104,9 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
     }
 
     for (i = 0; i < options->print_keys_count; i++) {
-        size_t len = MAX_STRING_LEN;
-        ret        = GRIB_SUCCESS;
+        size_t len  = MAX_STRING_LEN;
+        int keyType = options->print_keys[i].type;
+        ret         = GRIB_SUCCESS;
 
         if (h->product_kind == PRODUCT_BUFR) {
             /* ECC-236: Do not use grib_is_missing for BUFR */
@@ -1103,9 +1116,9 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
                 ret = grib_get_size(h, options->print_keys[i].name, &num_vals);
             }
             if (ret == GRIB_SUCCESS) {
-                if (options->print_keys[i].type == GRIB_TYPE_UNDEFINED)
-                    grib_get_native_type(h, options->print_keys[i].name, &(options->print_keys[i].type));
-                switch (options->print_keys[i].type) {
+                if (keyType == GRIB_TYPE_UNDEFINED)
+                    grib_get_native_type(h, options->print_keys[i].name, &keyType);
+                switch (keyType) {
                     case GRIB_TYPE_STRING:
                         acc = grib_find_accessor(h, options->print_keys[i].name);
                         ret = grib_get_string(h, options->print_keys[i].name, value, &len);
@@ -1134,7 +1147,7 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
                         ret = grib_get_string(h, options->print_keys[i].name, value, &len);
                         break;
                     default:
-                        fprintf(dump_file, "Could not determine type for %s\n", options->print_keys[i].name);
+                        fprintf(dump_file, "ERROR: Could not determine type for %s\n", options->print_keys[i].name);
                         exit(1);
                 }
             }
@@ -1146,9 +1159,9 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
             }
             else if (ret == GRIB_SUCCESS) {
                 const char* pName = NULL;
-                if (options->print_keys[i].type == GRIB_TYPE_UNDEFINED)
-                    grib_get_native_type(h, options->print_keys[i].name, &(options->print_keys[i].type));
-                switch (options->print_keys[i].type) {
+                if (keyType == GRIB_TYPE_UNDEFINED)
+                    grib_get_native_type(h, options->print_keys[i].name, &keyType);
+                switch (keyType) {
                     case GRIB_TYPE_STRING:
                         pName = options->print_keys[i].name;
                         if (fix_lsdate && strcmp(pName, "date") == 0) { /* ECC-707 */
@@ -1177,7 +1190,7 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
                         ret = grib_get_string(h, options->print_keys[i].name, value, &len);
                         break;
                     default:
-                        fprintf(dump_file, "invalid format option for %s\n", options->print_keys[i].name);
+                        fprintf(dump_file, "ERROR: Invalid format option for %s\n", options->print_keys[i].name);
                         exit(1);
                 }
             }
@@ -1245,7 +1258,7 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
             the_index = options->index;
             if (the_index >= size) {
                 fprintf(dump_file, "\n");
-                fprintf(stderr, "invalid index value %d (should be between 0 and %d)\n",
+                fprintf(stderr, "ERROR: Invalid index value %d (should be between 0 and %d)\n",
                         options->index, (int)(size - 1));
                 exit(1);
             }
