@@ -7,6 +7,9 @@
  * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
  * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
+#ifndef GRIB_TOOLS_H
+#define GRIB_TOOLS_H
+
 
 #ifdef __gnu_hurd__
 #define _FILE_OFFSET_BITS 64 /* 64-bit offsets off_t not the default on Hurd/i386 */
@@ -18,9 +21,6 @@
 #include <unistd.h>
 #endif
 
-#ifndef GRIB_TOOLS_H
-#define GRIB_TOOLS_H
-
 #ifndef S_IFMT
 #define S_IFMT 0170000 /*  type of file */
 #endif
@@ -33,10 +33,10 @@
 #define S_ISLNK(mode) (((mode) & (S_IFMT)) == (S_IFLNK))
 #endif
 
-#define MAX_KEYS 1000
+#define MAX_KEYS       256
 #define MAX_STRING_LEN 512
-#define MAX_FAILED 1024
-#define MAX_CONSTRAINT_VALUES 500
+#define MAX_FAILED     1024
+#define LATLON_SIZE    4 /* nearest */
 
 #define MODE_GRIB 0
 #define MODE_GTS 1
@@ -45,6 +45,10 @@
 #define MODE_TAF 5
 #define MODE_ANY 6
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef union grib_typed_value
 {
     long* long_value;
@@ -53,6 +57,7 @@ typedef union grib_typed_value
 } grib_typed_value;
 
 /*
+#define MAX_CONSTRAINT_VALUES 500
 typedef struct grib_constraints {
   const char* name;
   int         type;
@@ -140,15 +145,15 @@ typedef struct grib_runtime_options
     int gts;
     char* orderby;
     char* latlon;
-    double lats[4];
-    double lons[4];
-    double values[4];
-    double distances[4];
-    int indexes[4];
+    double lats[LATLON_SIZE];
+    double lons[LATLON_SIZE];
+    double values[LATLON_SIZE];
+    double distances[LATLON_SIZE];
+    int indexes[LATLON_SIZE];
     int latlon_mode;
     char* latlon_mask;
     int latlon_idx;
-    double mask_values[4];
+    double mask_values[LATLON_SIZE];
     int index;
     int index_on;
     double constant;
@@ -173,9 +178,9 @@ typedef struct grib_runtime_options
 
 extern grib_option grib_options[];
 extern int grib_options_count;
-extern const char* grib_tool_name;
-extern const char* grib_tool_description;
-extern const char* grib_tool_usage;
+extern const char* tool_name;
+extern const char* tool_description;
+extern const char* tool_usage;
 
 extern FILE* dump_file;
 
@@ -184,8 +189,8 @@ int grib_options_on(const char* id);
 int grib_options_get(int argc, char** argv);
 int grib_options_get_values(int argc, char** argv, int values_required, int default_type, grib_values values[], int* n, int* optind);
 int grib_tool(int argc, char** argv);
-char* grib_options_get_help(char* id);
-char* grib_options_get_args(char* id);
+const char* grib_options_get_help(const char* id);
+const char* grib_options_get_args(const char* id);
 int grib_options_command_line(const char* id);
 void usage(void);
 void usage_doxygen(void);
@@ -205,6 +210,10 @@ int grib_process_runtime_options(grib_context* c, int argc, char** argv, grib_ru
 void grib_tools_write_message(grib_runtime_options* options, grib_handle* h);
 int grib_tool_new_filename_action(grib_runtime_options* options, const char* file);
 int grib_no_handle_action(grib_runtime_options* options, int err);
-int exit_if_input_is_directory(const char* tool_name, const char* filename);
+int exit_if_input_is_directory(const char* toolname, const char* filename);
 
+#ifdef __cplusplus
+}
 #endif
+
+#endif /* GRIB_TOOLS_H */
